package extensions.awt;
import java.awt.*;
import java.util.*;

/** Construct a panel of ButtonActions.
  * @see extensions.awt.ButtonDialog
  * @version 1.0.2
  * @author John Webster Small
  */
public class ButtonActionPanel
  extends Panel
  implements ActionProtocol
{
  private ActionProtocol ap;
  private int clickedIdx = -1;

  /** Buttons to be oriented horizonally. */
  public final static int HORIZONAL = 1;

  /** Buttons to be oriented vertically, left justified. */
  public final static int VERT_LEFT = 2;

  /** Buttons to be oriented vertically, center justified. */
  public final static int VERT_CENTER = 3;

  /** Buttons to be oriented vertically, right justified. */
  public final static int VERT_RIGHT = 4;

  /** Horizonal pixel gap between buttons. */
  public static int hgap = 10;

  /** Vertical pixel gap between buttons. */
  public static int vgap = 5;

  /** enable/disable indexed button.
    * @param cond if true enable button
    * @param buttonIdx button to enable/disable
    */
  public void enable(boolean cond, int buttonIdx)
  {
    ((ButtonAction)getComponent(buttonIdx)).enable(cond);
  }

  /** If the object is a ButtonAction return its
    * index.
    * @param buttonCandidate
    * @return button index or -1 if not a member
    */
  public int buttonIdx(Object buttonCandidate)
  {
    if (buttonCandidate instanceof ButtonAction)
      for (int i = countComponents(); i-- > 0; )
        if (getComponent(i) == buttonCandidate)
        return i;
    return -1;
  }

  /** Does the specified button belong to this panel? */
  public boolean isMemberButton(ButtonAction ba)
  { return ((buttonIdx(ba) >= 0)? true : false); }

  /** Get specified button.
    * @param idx button index
    * @return returns button
    */
  public ButtonAction getButton(int idx)
    { return (ButtonAction) getComponent(idx); }

  /** Not called by user. */
  public void action(Component forward, Event evt, Object what)
  {
    clickedIdx = ((ButtonAction)evt.target).id;
    if (ap != null)
      ap.action(forward,evt,what);
  }

  /** User defined buttons are numbered starting at 0.
    * @return index of button last clicked or -1
    * @see #getClickedLabel
    */
  public int getClickedIdx()
    { return clickedIdx; }

  /** Programmatically click a button.
    * @param idx index of button clicked
    */
  public void setClickedIdx(int idx)
  {
    if (idx < countComponents() && idx >= -1)
      clickedIdx = idx;
  }

  /** Return the label from the button clicked. */
  public String getClickedLabel()
  {
     if (clickedIdx >= 0)
       return ((ButtonAction)getComponent(clickedIdx)).getLabel();
     return "";
  }

  /** Not called by user - reset clicked index.
    * @see #getClickedIdx
    */
  public void show()
  {
    clickedIdx = -1;
    super.show();
  }

  /** Request indicated button to receive focus.
    * @param idx button index
    */
  public void requestFocus(int idx)
    { getComponent(idx).requestFocus(); }

  /**
    * Construct a panel with ButtonActions.  Maintain
    * index of clicked button.
    * @param buttons labels of buttons separated by "|"
    * @param orientation HORIZONAL, VERT_LEFT,
    *   VERT_CENTER, or VERT_RIGHT
    * @param ap action handler for all buttons
    * @see extensions.awt.ActionProtocol
    */
  public ButtonActionPanel
    (String buttons, int orientation, ActionProtocol ap)
  {
    this.ap = ap;
    GridBagLayout gridbag = new GridBagLayout();
    GridBagConstraints c = new GridBagConstraints();
    setLayout(gridbag);
    c.fill = GridBagConstraints.NONE;
    if (orientation == HORIZONAL)  {
      c.gridheight = GridBagConstraints.REMAINDER;
    }
    else  {
      c.gridwidth = GridBagConstraints.REMAINDER;
      switch (orientation)  {
        case VERT_RIGHT:
          c.anchor = GridBagConstraints.EAST;
          break;
        case VERT_CENTER:
          c.anchor = GridBagConstraints.CENTER;
          break;
        case VERT_LEFT:
        default:
          c.anchor = GridBagConstraints.WEST;
      }
    }
    Button b;
    StringTokenizer bl = new StringTokenizer(buttons,"|");
    for (int i = 0; bl.hasMoreTokens(); i++)  {
      if (i == 0) // first
        c.insets = new Insets(vgap,hgap,vgap,hgap);
      else // all but first
        if (orientation == HORIZONAL)
          c.insets = new Insets(vgap,0,vgap,hgap);
        else
          c.insets = new Insets(0,hgap,vgap,hgap);
      b = new ButtonAction(bl.nextToken(),i,this);
      gridbag.setConstraints(b, c);
      add(b);
    }
    validate();
  }

  /**
    * Construct a panel with ButtonActions.  Maintain
    * index of clicked button.
    * @param buttons labels of buttons separated by "|"
    * @param orientation HORIZONAL, VERT_LEFT,
    *   VERT_CENTER, or VERT_RIGHT
    */
  public ButtonActionPanel
    (String buttons, int orientation)
    { this(buttons,orientation,null); }

  /**
    * Construct a panel with horizonal ButtonActions.
    * Maintain index of clicked button.
    * @param buttons labels of buttons separated by "|"
    * @param ap action handler for all buttons
    * @see extensions.awt.ActionProtocol
    */
  public ButtonActionPanel(String buttons, ActionProtocol ap)
    { this(buttons,HORIZONAL,ap); }

  /**
    * Construct a panel with horizonal ButtonActions.
    * Maintain index of clicked button.
    * @param buttons labels of buttons separated by "|"
    */
  public ButtonActionPanel(String buttons)
    { this(buttons,null); }
}
