package extensions.awt;
import java.awt.*;
import java.io.*;
import extensions.io.*;

class FileTextAreaRefreshAdapter extends Action
{
  private FileTextArea fta;
  FileTextAreaRefreshAdapter(FileTextArea fta) { this.fta = fta; }
  public final void run() { fta.refresh(); }
}

class FileTextAreaNewAdapter extends Action
{
  private FileTextArea fta;
  FileTextAreaNewAdapter(FileTextArea fta) { this.fta = fta; }
  public final void run() { fta.newFile(); }
}

class FileTextAreaBrowseAdapter extends Action
{
  private FileTextArea fta;
  FileTextAreaBrowseAdapter(FileTextArea fta) { this.fta = fta; }
  public final void run()  { fta.browse(); }
}

class FileTextAreaOpenAdapter extends Action
{
  private FileTextArea fta;
  FileTextAreaOpenAdapter(FileTextArea fta) { this.fta = fta; }
  public final void run()  { fta.open(); }
}

class FileTextAreaSaveAdapter extends Action
{
  private FileTextArea fta;
  FileTextAreaSaveAdapter(FileTextArea fta) { this.fta = fta; }
  public final void run()  { fta.save(); }
}

class FileTextAreaSaveAsAdapter extends Action
{
  private FileTextArea fta;
  FileTextAreaSaveAsAdapter(FileTextArea fta) { this.fta = fta; }
  public final void run()  { fta.saveAs(); }
}

class FileTextAreaInsertFileAdapter extends Action
{
  private FileTextArea fta;
  FileTextAreaInsertFileAdapter(FileTextArea fta) { this.fta = fta; }
  public final void run()  { fta.insertFile(false); }
}

class FileTextAreaAppendFileAdapter extends Action
{
  private FileTextArea fta;
  FileTextAreaAppendFileAdapter(FileTextArea fta) { this.fta = fta; }
  public final void run()  { fta.insertFile(true); }
}

/**
  * Extend TextArea, providing file based persistence.
  * @see java.awt.TextArea
  * @version 1.0.2
  * @author John Webster Small
  */
public class FileTextArea
  extends TextAreaExtended
  implements Cloneable
{
  private Frame parent;
  private FileExtended fe;

  /** Indicates a new file event. */  
  public static final int NEW_FILE = 0;

  /** Indicates a browse file event. */
  public static final int BROWSE = 1;

  /** Indicates an open file event. */
  public static final int OPEN = 2;

  /** Indicates a save file event. */
  public static final int SAVE = 3;

  /** Indicates what event has happened. */
  public int event = NEW_FILE;

  /** User canceled open file in constructor. */
  boolean userCanceled = false;

  /* Post the event to the FrameExtended. */
  private void changed(int event)
  {
    this.event = event;
    if (parent instanceof FrameExtended)
      ((FrameExtended)parent).update(this,fe.getName());
  }

  /** Return File. */
  public FileExtended getFile() { return fe; }

  /** Return directory/filename. */
  public String getAbsolutePath()
    { return fe.getAbsolutePath(); }

  private void init(Frame parent)
  {
    this.parent = parent;
    fe = new FileExtended();
    resetDirty();
    changed(NEW_FILE);
  }

  /**
    * Constructs a new FileTextArea.
    * @param parent Frame from which to launch a FileDialog
    */
  public FileTextArea(Frame parent) { init(parent); }

  /**
    * Constructs a new FileTextArea.
    * @param parent Frame from which to launch a FileDialog
    * @param rows number of lines of visible text
    * @param cols number of columns of visible text
    */
  public FileTextArea(Frame parent, int rows, int cols)
    { super(rows,cols); init(parent); }

  /**
    * Constructs a new FileTextArea.
    * @param parent Frame from which to launch a FileDialog
    * @param f File to load and view
    * @param readOnly true if browsing file only
    * @param promptUser allow user to modify file requested
    */
  public FileTextArea
    (Frame parent, File f, boolean readOnly, boolean promptUser)
  {
    this.parent = parent;
    fe = new FileExtended(f);
    resetDirty();
    userCanceled = !openFile(readOnly,promptUser);
  }

  /**
    * Constructs a new FileTextArea.
    * @param parent Frame from which to launch a FileDialog
    * @param text contents
    */
  public FileTextArea(Frame parent, String text)
    { super(text); init(parent); }

  /**
    * Constructs a new FileTextArea.
    * @param parent Frame from which to launch a FileDialog
    * @param text contents
    * @param rows number of lines of visible text
    * @param cols number of columns of visible text
    */
  public FileTextArea(Frame parent, String text, int rows, int cols)
    { super(text,rows,cols); init(parent); }

  /**
    * Copy initializer, set browse mode.
    * @param parent Frame from which to launch a FileDialog
    * @param fta FileTextArea to clone
    */
  public FileTextArea(Frame parent, FileTextArea fta)
  {
    super(fta.getText());
    setEditable(false);
    this.parent = parent;
    fe = new FileExtended(fta.fe);
    resetDirty();
    changed(BROWSE);
  }

  /** Clone the FileTextArea, its contents and
    * filename specs.
    */
  public Object clone()
    { return new FileTextArea(parent,this); }

  /** Save to file.
    * @param parent frame requesting the saving.
    * @param dirtyPrompt allow user to decline saving.
    * @param saveAsPrompt allow user to rename
    * @return false if canceled by user or if an
    *   exception is thrown
    */
  public boolean saveFile
    (Frame parent, boolean dirtyPrompt, boolean saveAsPrompt)
  {
    if (dirty() || saveAsPrompt) 
    {
       int idx = 0;
       if (dirtyPrompt)  {
         idx = MessageDialog.inquire
           (parent,fe.getName(),
            fe.getName()+ " has been modified!\nSave?",
            "Yes|No|Cancel");
       }
       if (idx == 0)  {
         try {
           if (saveAsPrompt || !fe.exists())  {
             FileExtended f = fe.save(parent,"",getText());
             if (f == null) return false;
             fe = f;
           }
           else
             fe.save(getText());
           resetDirty();
           changed(SAVE);
           return true;
         } catch (IOException ioe) {
           new DialogExtended(parent,ioe);
         }
       }
       else if (idx == 1)
         return true; // discard, i.e. as good as saved
       return false;  // cancel
    }
    return true;
  }

  /** Save to file, interactively. */
  public boolean saveAs()
    { return saveFile(parent,false,true); }

  /** Save to file.
    * @return false if canceled by user or if an
    *   exception is thrown
    */
  public boolean save()
    { return saveFile(parent,false,false); }

  /** Save to file.
    * @param parent frame requesting the save.
    * @return false if canceled by user or if an
    *   exception is thrown
    */
  public boolean save(Frame parent)
    { return saveFile(parent,false,false); }

  /** Refresh from file. */
  public void refresh()
  {
    try  {
      setText(fe.load());
      resetDirty();
    } catch (IOException ioe) {
      new DialogExtended(parent,ioe);
    }
  }

  /** Clear contents, if dirty. Allows user to cancel. */
  public void newFile()
  {
    if (saveFile(parent,true,false))  {
      setText("");
      resetDirty();
      fe = new FileExtended();
      changed(NEW_FILE);
    }
  }

  /** Open file, interactively, allowing user to cancel.
    * @param readOnly true if browsing file only
    * @param promptUser allow user to modify file requested
    * @return false if canceled by user or if an
    *   exception is thrown
    */  
  public boolean openFile(boolean readOnly, boolean promptUser)
  {
    if (saveFile(parent,true,false))  {
      try {
        if (promptUser)  {
          FileExtended f = fe.fileDialog(parent,"",FileDialog.LOAD);
          if (f == null)
            return false;
          fe = f;
        }
        if (fe.isFile())
          setText(fe.load());
        else
          setText("");
        resetDirty();
        setEditable(!readOnly);
        if (fe.isFile())
          changed(readOnly?BROWSE:OPEN);
        else
          changed(NEW_FILE);
        return true;
      } catch (IOException ioe) {
        new FrameExtended(ioe,0,true);
      }
    }
    return false;
  }

  /** Open file (interactively) in read only mode.
    * @return false if canceled by user or if an
    *   exception is thrown
    */
  public boolean browse()
    { return openFile(true,true); }

  /** Open file (interactively) for editing.
    * @return false if canceled by user or if an
    *   exception is thrown
    */
  public boolean open()
    { return openFile(false,true); }

  /** Insert file (interactively), allow user to cancel.
    * @param append if true append else insert at
    *   cursor location.
    * @return false if canceled by user or if an
    *   exception is thrown
    */
  public boolean insertFile(boolean append)
  {
    String newText;
    try {
      FileExtended f = fe.fileDialog(parent,"",FileDialog.LOAD);
      if (f != null)  {
        fe = f;
        if (append)
          appendText(fe.load());
        else
          replaceText(fe.load(),getSelectionStart(),getSelectionEnd());
        return true;
      }
    } catch (IOException ioe) {
      new DialogExtended(parent,ioe);
    }
    return false;
  }

  /**
    * Used to construct "Refresh" method adapter for action events.
    * @see extensions.awt.ActionProtocol
    */
  public ActionProtocol refreshAdapter()
   { return new FileTextAreaRefreshAdapter(this); }

  /**
    * Used to construct "newFile" method adapter for action events.
    * @see extensions.awt.ActionProtocol
    */
  public ActionProtocol newAdapter()
   { return new FileTextAreaNewAdapter(this); }

  /**
    * Used to construct "browse" method adapter for action events.
    * @see extensions.awt.ActionProtocol
    */
  public ActionProtocol browseAdapter()
   { return new FileTextAreaBrowseAdapter(this); }

  /**
    * Used to construct "open" method adapter for action events.
    * @see extensions.awt.ActionProtocol
    */
  public ActionProtocol openAdapter()
   { return new FileTextAreaOpenAdapter(this); }

  /**
    * Used to construct "save" method adapter for action events.
    * @see extensions.awt.ActionProtocol
    */
  public ActionProtocol saveAdapter()
   { return new FileTextAreaSaveAdapter(this); }

  /**
    * Used to construct "save as" method adapter for action events.
    * @see extensions.awt.ActionProtocol
    */
  public ActionProtocol saveAsAdapter()
   { return new FileTextAreaSaveAsAdapter(this); }

  /**
    * Used to construct "Insert file" method adapter for action events.
    * @see extensions.awt.ActionProtocol
    */
  public ActionProtocol insertFileAdapter()
   { return new FileTextAreaInsertFileAdapter(this); }

  /**
    * Used to construct "Append file" method adapter for action events.
    * @see extensions.awt.ActionProtocol
    */
  public ActionProtocol appendFileAdapter()
   { return new FileTextAreaAppendFileAdapter(this); }
}
