package extensions.awt;
import java.awt.*;
import java.io.*;

// Used for standard output and error TextAreas
class ProcessFrameTextArea extends TextArea
  implements Runnable
{
  private Frame parent;
  private Thread t;
  private InputStream in;
  
  void setText(InputStream in)
  {
    this.in = in;
    setText("");
    t = new Thread(this,"ProcessFrameTextArea");
    t.start();
  }

  void stop() { t.stop(); }

  void destroy() { t.stop(); }

  public void finalize() throws Throwable
    { destroy(); super.finalize(); }

  ProcessFrameTextArea(Frame parent)
    { this.parent = parent; }

  ProcessFrameTextArea(Frame parent, InputStream in)
  {
    this.parent = parent;
    setText(in);
  }

  public void run()
  {
    DataInputStream dis;
    try {
      dis = new DataInputStream(in);
      String s = dis.readLine();
      while(s != null)  {
        appendText("\n"+s);
        s = dis.readLine();
      }
    } catch (IOException ioe) {}
    finally  {
      dis = null;
    }
  }
}

/** Spawn process and view its standard output and error.
  * @version 1.0.2
  * @author John Webster Small
  */
public class ProcessFrame extends FrameExtended
  implements Runnable
{
  private String execCmd;
  protected ProcessFrameTextArea output, error;
  private Thread t;
  private Process p;

  /** Construct Viewing frame and start process,
    * monitoring its output and error streams.
    * @param title frame's title
    * @param execCmd the command to execute
    */
  public ProcessFrame(String title, String execCmd)
  {
    super("(Process) " + title);
    this.execCmd = execCmd;
    output = error = null;
    p = null;
    t = new Thread(this,"ProcessFrame");
    MenuBar mb = new MenuBar();
    Menu m = new Menu("File");
    m.add(new MenuItemAction("Exit",destroyAdapter()));
    m.add(new MenuItemAction("Exit All",destroyFramesAdapter()));
    mb.add(m);
    m = new Menu("Windows");
    m.add(new MenuItemAction("List ...",pickFrameAdapter()));
    m.add(new MenuItemAction("Cascade",cascadeFramesAdapter()));
    mb.add(m);
    setMenuBar(mb);
    t.start();
  }

  /** Wait for process to complete. */
  public void waitFor() throws InterruptedException
    { t.join(); }

  private void stopProcess()
  {
    if (p != null)  {
      if (output != null)
        output.stop();
      if (error != null)
        error.stop();
      p.destroy();
      p = null;
    }
  }

  /** Destroy this ProcessFrame. */
  public void destroy()
  {
    stopProcess();
    t.stop();
    super.destroy();
  }

  /** Destructor. */
  public void finalize() throws Throwable
    { destroy(); super.finalize(); }

  /** Did the process produce output? */
  public boolean isOutput()
  {
    return (output.getText() != null && output.getText().length() > 0);
  }

  /** Did the process produce any errors? */
  public boolean isError()
  {
    return (error.getText() != null && error.getText().length() > 0);
  }

  /** Run thread that starts both output and error threads. */
  public void run()
  {
    ChoiceCardPanel ccp = new ChoiceCardPanel();
    ccp.add("Standard Output",
      output=new ProcessFrameTextArea(this));
    ccp.add("Standard Error",
      error=new ProcessFrameTextArea(this));
    ccp.add("Command",new TextArea(execCmd));
    add("Center",ccp);
    pack();
    show();
    try {
      p = Runtime.getRuntime().exec(execCmd);
      output.setText(p.getInputStream());
      error.setText(p.getErrorStream());
      int cursor = getCursorType();
      try {
        setCursor(WAIT_CURSOR);
        p.waitFor();
      } finally { setCursor(cursor); }
    } catch (IOException ioe)  {
      new DialogExtended(this,ioe,true);
    } catch (InterruptedException ie)  {
      new DialogExtended(this,ie,true);
    } finally {
      stopProcess();
      if (isError())
        ccp.show("Standard Error");
    }
  }
}

