package extensions.awt;
import java.awt.*;

/**
  * Generic prompt modal dialog.
  * @version 1.0.2
  * @author John Webster Small
  */
public class PromptDialog extends ButtonDialog
  implements ActionProtocol
{
  private TextFieldAction responseField;
  private String defaultResponse = null;

  /** Do not call. */
  public boolean update(Object anySubject, Object what)
  {
    if (anySubject instanceof Event)
      if (isMemberButton(((Event)anySubject).target))
        if (getButtonClickedIdx() == 1 && defaultResponse != null)  {
          responseField.setText(defaultResponse);
          return true;
        }
    return super.update(anySubject,what);
  }

  /** User response. */
  public String response = null;

  /**
    * Constructs a modal PromptDialog.  Assign to
    * "response" before calling show() to preload field.
    * @param parent dialog's parent frame
    * @param title dialog's title
    * @param defaultResponse default response causes default
    *   button to appear
    * @param cols width of response field
    */
  public PromptDialog
    (Frame parent, String title, String defaultResponse, int cols)
  {
    super(parent,title,
      ((defaultResponse == null || defaultResponse.length() == 0)?
        OkCancel : OkDefaultCancel));
    Panel p = new Panel();
    p.setLayout(new FlowLayout(FlowLayout.CENTER,15,15));
    if (defaultResponse == null || defaultResponse.length() == 0)
      if (cols > 0)
        responseField = new TextFieldAction(cols,this);
      else
        responseField = new TextFieldAction(this);
    else  {
      this.defaultResponse = defaultResponse;
      if (cols > 0)
        responseField = new TextFieldAction(defaultResponse,cols,this);
      else
        responseField = new TextFieldAction(defaultResponse,this);
    }
    p.add(responseField);
    add("Center",p);
    pack();
  }

  /**
    * Constructs a modal PromptDialog.  Assign to
    * "response" before calling show() to preload field.
    * @param parent dialog's parent frame
    * @param title dialog's title
    * @param defaultResponse default response causes default
    *   button to appear
    */
  public PromptDialog
    (Frame parent, String title, String defaultResponse)
    { this(parent,title,defaultResponse,0); }

  /**
    * Constructs a modal PromptDialog.  Assign to
    * "response" before calling show() to preload field.
    * @param parent dialog's parent frame
    * @param title dialog's title
    */
  public PromptDialog(Frame parent, String title)
    { this(parent,title,null,0); }

  /**
    * Constructs a modal PromptDialog.  Assign to
    * "response" before calling show() to preload field.
    * @param parent dialog's parent frame
    * @param title dialog's title
    * @param cols width of response field
    */
  public PromptDialog
    (Frame parent, String title, int cols)
    { this(parent,title,null,cols); }

  /** Preload field with response. */
  public void show()
  {
    if (response != null)
      responseField.setText(response);
    else
      responseField.setText("");
    responseField.requestFocus();
    responseField.selectAll();
    super.show();
    response = responseField.getText();
  }

  /** If okay clicked? */
  public boolean okClicked()
    { return getButtonClickedIdx() == 0; }

  /** Do not call.
    * @see java.awt.Component#action
    */
  public final void action(Component forward, Event evt, Object what)
  {
    if (evt.target == responseField)  {
      setClickedIdx(0);  // programmatically click "OK"
      destroy();
    }
    else
      super.action(forward,evt,what);
  }

  /** If the user clicks ok or presses carriage return
    * the indicated property is set to pd.response.
    * Allows another dialog to call a prompt dialog while
    * working around AWT modal bug on JDK 1.0.2 win32.
    * @param pd the instantiated prompt dialog that will
    *   be shown
    * @param pp the property that will be set
    * @param focusReturn the component to receive focus
    *   upon returning from dialog
    */
  public static void setProperty
    (PromptDialog pd, PropertyProtocol pp, Component focusReturn)
    { new PromptDialogSetPropertyThread(pd,pp,focusReturn); }
}

class PromptDialogSetPropertyThread
  extends Thread
{
  private PromptDialog pd;
  private PropertyProtocol pp;
  private Component focusReturn;

  PromptDialogSetPropertyThread
    (PromptDialog pd, PropertyProtocol pp, Component focusReturn)
  {
    super("PromptDialogSetPropertyThread");
    this.pd = pd;
    this.pp = pp;
    this.focusReturn = focusReturn;
    start();
  }

  public void run()
  {
    pd.show();
    if (pd.okClicked())
      pp.set(pd.response);
//    focusReturn.requestFocus();
  }
}
