package extensions.awt;
import java.awt.*;
import java.util.*;

class TextAreaExtendedShowFindDialogAdapter extends Action
{
  private TextAreaExtended tae;
  TextAreaExtendedShowFindDialogAdapter(TextAreaExtended tae)
    { this.tae = tae; }
  public final void run()
    { tae.showFindDialog(FrameExtended.getParentFrame(forward)); }
}

class TextAreaExtendedShowReplaceDialogAdapter extends Action
{
  private TextAreaExtended tae;
  TextAreaExtendedShowReplaceDialogAdapter(TextAreaExtended tae)
    { this.tae = tae; }
  public final void run()
    { tae.showReplaceDialog(FrameExtended.getParentFrame(forward)); }
}

class TextAreaExtendedBookmarkAdapter implements ActionProtocol
{
  private TextAreaExtended tae;
  TextAreaExtendedBookmarkAdapter(TextAreaExtended tae)
    { this.tae = tae; }
  public final void action(Component forward, Event evt, Object what)
    { tae.bookmark(); }
}

class TextAreaExtendedGotoBookmarkAdapter implements ActionProtocol
{
  private TextAreaExtended tae;
  TextAreaExtendedGotoBookmarkAdapter(TextAreaExtended tae)
    { this.tae = tae; }
  public final void action(Component forward, Event evt, Object what)
    { tae.gotoBookmark(); }
}

/**
  * Construct TextAreaExtended which maintains dirty
  * bit and cursor positioning methods.
  * @version 1.0.2
  * @author John Webster Small
  */
public class TextAreaExtended extends TextArea
{
  private int[] rowMarkers = null;
  private String original = "";
  private int bookmarkPos = -1;
  private TextAreaExtendedFindDialog findDialog = null;
  private TextAreaExtendedReplaceDialog replaceDialog = null;
  private int selectionStart = 0;
  private int selectionEnd = 0;

  /** Constructs a new TextAreaExtended. */
  public TextAreaExtended()
    { super(); }

  /**
    * Constructs a new TextAreaExtended.
    * @param rows number of lines of visible text
    * @param cols number of columns of visible text
    */
  public TextAreaExtended(int rows, int cols)
    { super(rows,cols); }

  /**
    * Constructs a new TextAreaExtended.
    * @param text contexts
    */
  public TextAreaExtended(String text)
  {
    super(text);
    original = text;
  }

  /**
    * Constructs a new TextAreaExtended.
    * @param text contexts
    * @param rows number of lines of visible text
    * @param cols number of columns of visible text
    */
  public TextAreaExtended(String text, int rows, int cols)
  {
    super(text,rows,cols);
    original = text;
  }

  /** Are the contents dirty?  */
  public boolean dirty()
  {
    String current = getText();
    if (original != current)
        if (original != null)
          if (original.compareTo(current) != 0)
            return true;
    return false;
  }

  /** Reset dirty bit. */
  public void resetDirty()
  {
    markRows();
    original = getText();
  }

  /** Discard modifications. */
  public void restoreOriginal()
    { setText(original); }

  private void markRows()
  {
    Vector v = new Vector();
    int idx = 0;
    String t = getText();
    while (idx < t.length())  {
      idx = t.indexOf('\n',idx);
      if (idx < 0)
        break;
      v.addElement(new Integer(++idx));
    }
    rowMarkers = new int[(v.size() > 0)?v.size():1];
    int i;
    for (i = 0; i < v.size(); i++)
      rowMarkers[i] = ((Integer)v.elementAt(i)).intValue();
    while (i < rowMarkers.length)
      rowMarkers[i++] = 0;
  }

  /**
    * Position cursor. Top-left is 0.
    * @pos linear position of cursor
    */
  public void gotoPos(int pos)
  {
    boolean editMode = isEditable();
    if (pos < getText().length())
      insertText("",pos);
    else
      appendText("");
    setEditable(editMode);
  }

  /**
    * Position cursor.  Top-left is (1,1)
    * @param row cursor row
    * @param col cursor column
    */
  public void gotoRC(int row, int col)
  {
    if (dirty() || rowMarkers == null)
      markRows();
    if (row < 1)
      row = 1;
    row -= 2;  // first row always begins at pos == 0
    if (row >= rowMarkers.length)
      row = rowMarkers.length - 1;
    int pos;
    if (row >= 0)
      pos = rowMarkers[row];
    else
      pos = 0;
    int nextPos;
    if (++row >= rowMarkers.length)
      nextPos = getText().length();
    else
      nextPos = rowMarkers[row];
    if (--col < 0)
      col = 0;
    pos += col;
    if (pos > nextPos)
      pos = nextPos;
    gotoPos(pos);
  }

  /**
    * Position cursor at the start of row.
    * Top line is row 1.
    * @param row cursor row
    */
  public void gotoRow(int row)
    { gotoRC(row,0); }

  /** Bookmark. */
  public void bookmark()
    { bookmarkPos = getSelectionEnd(); }

  /** Go to bookmark. */
  public void gotoBookmark()
  {
    if (bookmarkPos >= 0)
      gotoPos(bookmarkPos);
  }

  /** Find pattern in text.
    * @param pattern pattern to find
    * @param matchCase case sensitive?
    * @param down look after cursor?
    * @param fromIdx position from which to start looking
    * @return returns true if found
    */
  public boolean find
    (String pattern, boolean matchCase, boolean down, int fromIdx)
  {
    if (pattern.length() == 0)
      return false;
    String patternLC = pattern.toLowerCase();
    String textLC = getText().toLowerCase();
    String candidate;
    saveSelection();
    while (true)  {
      try  {
        if (down)
          fromIdx = textLC.indexOf(patternLC,fromIdx);
        else  {
          fromIdx = textLC.lastIndexOf(patternLC,fromIdx);
        }
        candidate =
          getText().substring(fromIdx,fromIdx+pattern.length());
        if ((matchCase?
          candidate.equals(pattern):
          candidate.equalsIgnoreCase(pattern)))  {
          requestFocus();
          select(fromIdx,fromIdx+pattern.length());
          saveSelection();
          return true;
        }
        if (down)
          fromIdx++;
        else
          fromIdx--;
      } catch(Throwable t) {
        requestFocus();
        restoreSelection();
        return false;
      }
    }
  }

  /** Show FindDialog for this TextAreaExtended.
    * Lazily construct as required.
    * @param parent  parent frame of TextAreaExtended
    */
  public void showFindDialog(Frame parent)
  {
    if (findDialog == null)
      findDialog = new TextAreaExtendedFindDialog(parent,this);
    findDialog.show();
  }

  /** Find pattern in text.
    * @param pattern pattern to find
    * @param replacement replacement text
    * @param matchCase case sensitive?
    * @param replaceAll replace all ocurrences from current
    *   cursor location onward or if false just the next
    *   ocurrence.
    * @return returns true if found
    */
  public boolean replace(String pattern, String replacement,
    boolean matchCase, boolean replaceAll)
  {
    boolean okay;
    int fromIdx;
    do  {
      fromIdx = getSelectionEnd();
      if (matchCase)  {
        if (getSelectedText().equals(pattern))
          replaceText(replacement,getSelectionStart(),getSelectionEnd());
      }
      else if (getSelectedText().equalsIgnoreCase(pattern))
        replaceText(replacement,getSelectionStart(),getSelectionEnd());
      okay = find(pattern,matchCase,true,fromIdx);
    } while (replaceAll && okay);
    return okay;
  }

  /** Show replace dialog for this TextAreaExtended.
    * Lazily construct as required.
    * @param parent  parent frame of TextAreaExtended
    */
  public void showReplaceDialog(Frame parent)
  {
    if (replaceDialog == null)
      replaceDialog = new TextAreaExtendedReplaceDialog(parent,this);
    replaceDialog.show();
  }

  /** Save current selection.
    * @see #restoreSelection
    */
  public void saveSelection()
  {
    selectionStart = getSelectionStart();
    selectionEnd = getSelectionEnd();
  }

  /** Restore saved selection.
    * @see #saveSelection
    */
  public void restoreSelection()
    { select(selectionStart,selectionEnd); }

  /** Causes the focus to shift to the overloaded
    * requestFocus() indicated component whenever
    * the frame is resized.
    * @see java.awt.Component#resize
    */
  public void resize(int width, int height)
  {
    saveSelection();  // JDK 1.0.2 win32 buggy
    super.resize(width,height);
    restoreSelection();
  }

  /** Causes the focus to shift to the overloaded
    * requestFocus() indicated component whenever
    * the frame is resized.
    * @see java.awt.Component#resize
    */
  public void resize(Dimension d)
  {
    saveSelection();  // JDK 1.0.2 win32 buggy
    super.resize(d);
    restoreSelection();
  }

  /** Causes the focus to shift to the overloaded
    * requestFocus() indicated component whenever
    * the frame is reshaped.
    * @see java.awt.Component#reshape
    */
  public void reshape(int x, int y, int width, int height)
  {
    saveSelection();  // JDK 1.0.2 win32 buggy
    super.reshape(x,y,width,height);
    restoreSelection();
  }

  /**
    * Used to construct "showFindDialog" method adapter for action events.
    * @see extensions.awt.ActionProtocol
    */
  public ActionProtocol showFindDialogAdapter()
  { return new TextAreaExtendedShowFindDialogAdapter(this); }
 
  /**
    * Used to construct "showReplaceDialog" method adapter for action events.
    * @see extensions.awt.ActionProtocol
    */
  public ActionProtocol showReplaceDialogAdapter()
  { return new TextAreaExtendedShowReplaceDialogAdapter(this); }

  /**
    * Used to construct "Bookmark" method adapter for action events.
    * @see extensions.awt.ActionProtocol
    */
  public ActionProtocol bookmarkAdapter()
   { return new TextAreaExtendedBookmarkAdapter(this); }

  /**
    * Used to construct "Go to bookmark" method adapter for action events.
    * @see extensions.awt.ActionProtocol
    */
  public ActionProtocol gotoBookmarkAdapter()
   { return new TextAreaExtendedGotoBookmarkAdapter(this); }
}

final class TextAreaExtendedFindDialog extends DialogExtended
  implements ActionProtocol
{
  private TextAreaExtended tae;
  private TextFieldActionPanel find;
  private ButtonActionPanel bap;
  private Checkbox matchCase, down;
  private int lastUpFindIdx = -1;

  TextAreaExtendedFindDialog(Frame parent, TextAreaExtended tae)
  {
    super(parent,"Find",false);
    disposeOnDestroy = false;
    this.tae = tae;
    add("Center",find = new TextFieldActionPanel("Find what:",30,this));
    add("East",bap = new ButtonActionPanel
      ("Find next|Cancel",ButtonActionPanel.VERT_LEFT,this));
    Panel p = new Panel();
    p.add(matchCase = new Checkbox("Match case"));
    CheckboxGroupPanel cgp = new CheckboxGroupPanel("Direction");
    cgp.add(new Checkbox("up"));
    cgp.add(down = new Checkbox("down"));
    down.setState(true);
    cgp.validate();
    p.add(cgp);
    add("South",p);
    pack();
  }

  public final void action(Component forward, Event evt, Object what)
  {
    int bidx = bap.buttonIdx(evt.target);
/*
    if (bidx < 0)
      if (evt.target instanceof TextField)
        bidx = 0;  // User pressed <ENTER> in text field
*/
    switch (bidx)  {
      case 0:  // find next
        String pattern = find.getTextField().getText();
        int fromIdx = tae.getSelectionEnd();
        if (!down.getState() && (fromIdx == lastUpFindIdx))  {
          fromIdx -= pattern.length();
          fromIdx--;
        }
        if (!tae.find(pattern,matchCase.getState(),
          down.getState(),fromIdx))  {
            MessageDialog.notifyUser
              (forward,"Jide","Cannot find: '"+pattern+"'");
        }
        else  {
          if (!down.getState())
            lastUpFindIdx = tae.getSelectionEnd();
        }
        break;
      case 1:  // cancel
        destroy();
        tae.requestFocus();
        break;
    }
  }

  public void show()
  {
    find.getTextField().requestFocus();
    find.getTextField().selectAll();
    lastUpFindIdx = -1;
    super.show();
  }
}

final class TextAreaExtendedReplaceDialog extends DialogExtended
  implements ActionProtocol
{
  private TextAreaExtended tae;
  private TextFieldActionPanel find, replace;
  private ButtonActionPanel bap;
  private Checkbox matchCase;

  TextAreaExtendedReplaceDialog(Frame parent, TextAreaExtended tae)
  {
    super(parent,"Replace",false);
    disposeOnDestroy = false;
    this.tae = tae;


    Panel p = new Panel();
    GridBagLayout gridbag = new GridBagLayout();
    GridBagConstraints c = new GridBagConstraints();
    p.setLayout(gridbag);
    c.fill = GridBagConstraints.NONE;
    c.gridwidth = GridBagConstraints.REMAINDER;
    c.anchor = GridBagConstraints.WEST;
    int vgap = 15; int hgap = 15;
    c.insets = new Insets(vgap,hgap,vgap,hgap);
    find = new TextFieldActionPanel("Find what:",30,this);
    gridbag.setConstraints(find, c);
    p.add(find);
    c.insets = new Insets(0,hgap,vgap,hgap);
    replace = new TextFieldActionPanel("Replace with:",30,this);
    gridbag.setConstraints(replace, c);
    p.add(replace);
    add("Center",p);
    add("East",bap = new ButtonActionPanel
      ("Find next|Replace|Replace All|Cancel",
      ButtonActionPanel.VERT_LEFT,this));
    p = new Panel();
    p.add(matchCase = new Checkbox("Match case"));
    add("South",p);
    pack();
  }

  public final void action(Component forward, Event evt, Object what)
  {
    int bidx = bap.buttonIdx(evt.target);
    String pattern = find.getTextField().getText();
    switch (bidx)  {
      case 0:  // find next
        int fromIdx = tae.getSelectionEnd();
        if (!tae.find(pattern,matchCase.getState(),true,fromIdx))  {
            MessageDialog.notifyUser
              (forward,"Jide","Cannot find: '"+pattern+"'");
        }
        break;
      case 1:  // replace
        if (!tae.replace(pattern,
            replace.getTextField().getText(),
            matchCase.getState(),false))  {
            MessageDialog.notifyUser
              (forward,"Jide","Cannot find: '"+pattern+"'");
        }
        break;
      case 2:  // replace all
        tae.replace(pattern,replace.getTextField().getText(),
            matchCase.getState(),true);
        break;
      case 3:  // cancel
        destroy();
        tae.requestFocus();
        break;
    }
  }

  public void show()
  {
    find.getTextField().requestFocus();
    find.getTextField().selectAll();
    super.show();
  }
}
