package extensions.awt;
import java.awt.*;
import java.io.*;

/** Display thread(group)s.
  * @version 1.0.2
  * @author John Webster Small
  */
public class ThreadFrame extends TextFrame
  implements Runnable
{

  private Thread refreshThread;

  /** Construct a ThreadFrame for viewing current thread(group)s. */
  public ThreadFrame()
  {
    super("Threads");
    refreshThread = new Thread(this,"ThreadFrame Refresh");
    refreshThread.start();
  }

  /** Run background thread to update ThreadFrame snapshot. */
  public void run()
  {
    while (true) {
      refresh();
      try { refreshThread.sleep(1000); }
      catch (Throwable t) { break; }
    }
  }

  /** Recursive print thread(group)s.
    * @param out PrintStream to list threads
    * @param thread a Thread or ThreadGroup instance
    * @param indent recursive indenting - set to ""
    */
  void printOn(PrintStream out, Object thread, String indent)
  {
    if (thread == null)
      return ;
    if (thread instanceof Thread)  {
      Thread t = (Thread) thread;
      out.println(indent+"Thread: "+t.getName()+
        " Priority: "+t.getPriority()+
        (t.isDaemon()?" Daemon":"")+
        (t.isAlive()?"":" Died"));
    }
    else if (thread instanceof ThreadGroup)  {
      ThreadGroup tg = (ThreadGroup) thread;
      out.println(indent+"Thread: "+tg.getName()+
        " Max Priority: "+tg.getMaxPriority()+
        (tg.isDaemon()?" Daemon":""));
      Thread[] threads = new Thread[tg.activeCount()];
      tg.enumerate(threads,false);
      for (int i = 0; i < threads.length; i++)
        printOn(out,threads[i],indent+"  ");
      ThreadGroup[] groups = new ThreadGroup[tg.activeGroupCount()];
      tg.enumerate(groups,false);
      for (int i = 0; i < groups.length; i++)
        printOn(out,groups[i],indent+"  ");
    }
  }

  /** Return root ThreadGroup of process.
    * @param tg any thread group or null
    * @return returns root thread group of process
    */
  public ThreadGroup rootThreadGroup(ThreadGroup tg)
  {
    ThreadGroup ptg;
    if (tg == null)
      return rootThreadGroup(Thread.currentThread().getThreadGroup());
    else if ((ptg = tg.getParent()) != null)
      return rootThreadGroup(ptg);
    else
      return tg;
  }

  /** Take new thread snapshot. */
  public void refresh()
  {
    TextArea ta = getTextArea();
    ta.setText("");
    ByteArrayOutputStream os = new ByteArrayOutputStream();
    printOn(new PrintStream(os),rootThreadGroup(null),"");
    ta.setText(os.toString());
  }

  /** Show the ThreadFrame after refreshing thread(group) info. */
  public void show()
  { 
    refresh();
    super.show();
  }

  /** Destroy the ThreadFrame */
  public void destroy()
  {
    refreshThread.stop();
    super.destroy();
  }

  /** For stand alone app. */
  public static void main(String[] args)
    { AppFrame.winMain("extensions.awt.ThreadFrame"); }
}
