package extensions.util;
import java.util.*;

/**
  * Maintain a class instance set.  instanceSet
  * are issued IDs.  Use in a class constructor
  * and dispose() methods to maintain a list
  * of all instanceSet in use.
  * <pre><tt>
  *
  *   class Foo
  *   {
  *     static Instances foos = new Instances();
  *     public Foo()
  *     {
  *       foos.getID(this); // register instance
  *       ...
  *     }
  *     public void dispose()
  *     {
  *       foos.dispose(this);
  *       ...
  *
  * </tt></pre>
  * @version 1.0.2
  * @author John Webster Small
  */
public class Instances
{
  private Hashtable instanceSet;
  private IDs ids;

  /** Construct an instance set. */
  public Instances()
  {
    instanceSet = new Hashtable();
    ids = new IDs();
  }

  /**
    * Register or lookup an instance.
    * @return returns ID of instance or 0 if not found
    */
  public int getID(Object instance)
  {
    int id = 0;

    if (instanceSet.contains(instance))  {
      Enumeration keys = instanceSet.keys();
      Integer key;
      while (keys.hasMoreElements())
        if (instance == instanceSet
          .get(key=(Integer)keys.nextElement()))  {
          id = key.intValue();
          break;
        }
    }
    else
      instanceSet.put(new Integer(id = ids.issue()),instance);
    return id;
  }

  /**
    * Find instance by its ID.
    * @param id ID of instance
    * @return returns instance associated with id
    */
  public Object getInstance(int id)
    { return instanceSet.get(new Integer(id)); }

  /**
    * Instance is being disposed of - remove instance
    * associated with ID from the instance set.
    * @param id ID of instance being disposed of
    */
  public void dispose(int id)
  {
    if (instanceSet.remove(new Integer(id)) != null)
      ids.recycle(id);
  }

  /**
    * Instance is being disposed of - remove instance
    * from instance set.
    * @param instance instance being disposed
    */
  public void dispose(Object instance)
    { dispose(getID(instance)); }

  /**
    * Return all current instances via a Select "all"
    * Enumeration isolator, e.g. instances can be
    * subsequently disposed() without effecting this
    * instances snapshoot!
    * @return Enumeration of instances
    */
  public Enumeration instances()
    { return new InstancesSibblings(null,instanceSet.elements()); }

  /**
    * @return Enumeration of sibbling instances
    */
  public Enumeration sibblings(Object thisInstance)
    { return new InstancesSibblings(thisInstance,instanceSet.elements()); }

  /** @return count of instances outstanding */
  public int size()
    { return instanceSet.size(); }
}

final class InstancesSibblings extends Select
{
  private Object thisInstance;

  protected boolean accept(Object candidate)
    { return candidate != thisInstance; }

  InstancesSibblings(Object thisInstance, Enumeration candidates)
  {
    super(candidates);
    this.thisInstance = thisInstance;
    init();
  }
}
